@@ -34,13 +34,17 @@ __all__=[
     'prune_parameter_in_channels',
     'prune_multihead_attention_out_channels',
     'prune_multihead_attention_in_channels',
+    'prune_groupnorm_out_channels',
+    'prune_groupnorm_in_channels',
+    'prune_instancenorm_out_channels',
+    'prune_instancenorm_in_channels',
 ]
 
 class BasePruningFunc(ABC):
     TARGET_MODULES = ops.TORCH_OTHERS  # None
 
-    def __init__(self, dim=1):
-        self.dim = dim
+    def __init__(self, pruning_dim=1):
+        self.pruning_dim = pruning_dim
 
     @abstractclassmethod
     def prune_out_channels(self, layer: nn.Module, idxs: Sequence[int]):
@@ -194,15 +198,15 @@ class BatchnormPruner(BasePruningFunc):
 class LayernormPruner(BasePruningFunc):
     TARGET_MODULES = ops.TORCH_LAYERNORM
 
-    def __init__(self, metrcis=None, dim=-1):
+    def __init__(self, metrcis=None, pruning_dim=-1):
         super().__init__(metrcis)
-        self.dim = dim
+        self.pruning_dim = pruning_dim
 
     def check(self, layer, idxs):
-        layer.dim = self.dim
+        layer.dim = self.pruning_dim
 
     def prune_out_channels(self, layer: nn.Module, idxs: Sequence[int]) -> nn.Module:
-        pruning_dim = self.dim
+        pruning_dim = self.pruning_dim
         if len(layer.normalized_shape) < -pruning_dim:
             return layer
         num_features = layer.normalized_shape[pruning_dim]
@@ -224,10 +228,49 @@ class LayernormPruner(BasePruningFunc):
     prune_in_channels = prune_out_channels
 
     def get_out_channels(self, layer):
-        return layer.normalized_shape[self.dim]
+        return layer.normalized_shape[self.pruning_dim]
 
     def get_in_channels(self, layer):
-        return layer.normalized_shape[self.dim]
+        return layer.normalized_shape[self.pruning_dim]
+
+class GroupNormPruner(BasePruningFunc):
+    def prune_out_channels(self, layer: nn.PReLU, idxs: list) -> nn.Module:
+        keep_idxs = list(set(range(layer.num_channels)) - set(idxs))
+        keep_idxs.sort()
+        layer.num_channels = layer.num_channels-len(idxs)
+        if layer.affine:
+            layer.weight = torch.nn.Parameter(
+                layer.weight.data.clone()[keep_idxs])
+            layer.bias = torch.nn.Parameter(layer.bias.data.clone()[keep_idxs])
+        return layer
+    
+    prune_in_channels = prune_out_channels
+
+    def get_out_channels(self, layer):
+        return layer.num_channels
+
+    def get_in_channels(self, layer):
+        return layer.num_channels
+
+class InstanceNormPruner(BasePruningFunc):
+    def prune_out_channels(self, layer: nn.Module, idxs: Sequence[int]) -> nn.Module:
+        keep_idxs = list(set(range(layer.num_features)) - set(idxs))
+        keep_idxs.sort()
+        layer.num_features = layer.num_features-len(idxs)
+        if layer.affine:
+            layer.weight = torch.nn.Parameter(
+                layer.weight.data.clone()[keep_idxs])
+            layer.bias = torch.nn.Parameter(layer.bias.data.clone()[keep_idxs])
+        return layer
+
+    prune_in_channels = prune_out_channels
+
+    def get_out_channels(self, layer):
+        return layer.num_features
+
+    def get_in_channels(self, layer):
+        return layer.num_features
+
 
 class PReLUPruner(BasePruningFunc):
     TARGET_MODULES = ops.TORCH_PRELU
@@ -330,23 +373,23 @@ class LSTMPruner(BasePruningFunc):
 
 class ParameterPruner(BasePruningFunc):
     TARGET_MODULES = ops.TORCH_PARAMETER
-    def __init__(self, dim=-1):
-        super().__init__(dim=dim)
+    def __init__(self, pruning_dim=-1):
+        super().__init__(pruning_dim=pruning_dim)
         
     def prune_out_channels(self, tensor, idxs: list) -> nn.Module:
-        keep_idxs = list(set(range(tensor.data.shape[self.dim])) - set(idxs))
+        keep_idxs = list(set(range(tensor.data.shape[self.pruning_dim])) - set(idxs))
         keep_idxs.sort()
         tensor.data = torch.index_select(
-            tensor.data, self.dim, torch.LongTensor(keep_idxs))
+            tensor.data, self.pruning_dim, torch.LongTensor(keep_idxs).to(tensor.device))
         return tensor
 
     prune_in_channels = prune_out_channels
 
     def get_out_channels(self, parameter):
-        return parameter.shape[self.dim]
+        return parameter.shape[self.pruning_dim]
 
     def get_in_channels(self, parameter):
-        return parameter.shape[self.dim]
+        return parameter.shape[self.pruning_dim]
 
 
 class MultiheadAttentionPruner(BasePruningFunc):
@@ -430,10 +473,11 @@ PrunerBox = {
     ops.OPTYPE.EMBED: EmbeddingPruner(),
     ops.OPTYPE.PARAMETER: ParameterPruner(),
     ops.OPTYPE.MHA: MultiheadAttentionPruner(),
-    ops.OPTYPE.LSTM: LSTMPruner()
+    ops.OPTYPE.LSTM: LSTMPruner(),
+    ops.OPTYPE.GN: GroupNormPruner(),
+    ops.OPTYPE.IN: InstanceNormPruner(),
 }
 
-
 def is_out_channel_pruner(fn):
     return fn in tuple(p.prune_out_channels for p in PrunerBox.values())
 
@@ -472,3 +516,9 @@ prune_multihead_attention_in_channels = PrunerBox[ops.OPTYPE.MHA].prune_in_chann
 
 prune_lstm_out_channels = PrunerBox[ops.OPTYPE.LSTM].prune_out_channels
 prune_lstm_in_channels = PrunerBox[ops.OPTYPE.LSTM].prune_in_channels
+
+prune_groupnorm_out_channels = PrunerBox[ops.OPTYPE.GN].prune_out_channels
+prune_groupnorm_in_channels = PrunerBox[ops.OPTYPE.GN].prune_in_channels
+
+prune_instancenorm_out_channels = PrunerBox[ops.OPTYPE.IN].prune_out_channels
+prune_instancenorm_in_channels = PrunerBox[ops.OPTYPE.IN].prune_in_channels
\ No newline at end of file
